 //GEOSC 597 Project
 //Name: Yi Fang
 //Date: 12/1/2016
 //Function: Control the injection/refill of syringe pump
 
 //Declare pin functions on Redboard
#define stp 2
#define dir 3
#define MS1 4 
#define MS2 5
#define EN  6

//Declare variables for functions
char user_input;
int x;
int y;
int state;

void setup() 
{
  pinMode(stp, OUTPUT);
  pinMode(dir, OUTPUT);
  pinMode(MS1, OUTPUT);
  pinMode(MS2, OUTPUT);
  pinMode(EN, OUTPUT);
  
  resetEDPins(); //Set step, direction, microstep and enable pins to default states
  Serial.begin(9600); //Open Serial connection for debugging
  Serial.println("Yi Fang - Syringe Pump System (GEOSC597 Project)");
  Serial.println();
  //Print function list for user selection
  Serial.println("Operational Instruction:");
  Serial.println("Pumping Mode:        P1:Injection  P2:Refill  P3:Inject-Refill Cycling");
  Serial.println("Flow Rate Selection: (a):0.585ml/s  (b):0.293ml/s  (c):0.146ml/s  (d):0.074ml/s");  
  Serial.println("1. Inject@0.585ml/s");
  Serial.println("2. Refill@0.585ml/s");
  Serial.println("3. Inject@0.293ml/s");
  Serial.println("4. Refill@0.293ml/s");
  Serial.println("5. Inject@0.146ml/s");
  Serial.println("6. Refill@0.146ml/s"); 
  Serial.println("7. Inject@0.074ml/s");
  Serial.println("8. Refill@0.074ml/s"); 
  Serial.println("9. Cycled Injecting & Refilling"); 
  Serial.println();
}

//Main loop to control 
void loop() {
  while(Serial.available()){
      user_input = Serial.read(); //Read user input and trigger appropriate function
      digitalWrite(EN, LOW);      //Pull enable pin low to allow motor control
      
      if (user_input =='1')       //1. Inject: 0.585ml/s
      {
        StepForwardDefault();
      }
      
      else if(user_input =='2')   //2. Refill: 0.585ml/s
      {
        ReverseStepDefault();
      }

      else if (user_input =='3')  //3. Inject: 0.293ml/s
      {
        StepForwardHalf();
      }
      
      else if(user_input =='4')   //4. Refill: 0.293ml/s
      {
        ReverseStepHalf();
      }

      else if (user_input =='5')  //5. Inject: 0.146ml/s
      {
        StepForwardQuarter();
      }
      
      else if(user_input =='6')   //6. Refill: 0.146ml/s
      {
        ReverseStepQuarter();
      }

      else if (user_input =='7')  //7. Inject: 0.146ml/s
      {
        StepForwardSmallest();
      }
      
      else if(user_input =='8')   //8. Refill: 0.146ml/s
      {
        ReverseStepSmallest();
      }
      
      else if(user_input =='9')
      {
        CycledInjectRefill();
      }
     
      else
      {
        Serial.println("Invalid option entered.");
      }
      resetEDPins();
  }
}

//1. Injection flow at 0.585ml/s=================================================================
void StepForwardDefault()
{
  Serial.println("Injection flow at 0.585ml/s");
  digitalWrite(dir, LOW); //Pull direction pin low to move "forward"
  for(x= 1; x<17100; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    digitalWrite(stp,HIGH); //Trigger one step forward
    delay(1);
    digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
    delay(1);
  }
  Serial.println("Enter new option");
  Serial.println();
}

//2. Refill flow at 0.585ml/s==================================================================
void ReverseStepDefault()
{
  Serial.println("Refill flow at 0.585ml/s");
  digitalWrite(dir, HIGH); //Pull direction pin high to move in "reverse"
  for(x= 1; x<17200; x++)  //Loop the stepping enough times for motion to be visible
  {
    digitalWrite(stp,HIGH); //Trigger one step
    delay(1);
    digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
    delay(1);
  }
  Serial.println("Enter new option");
  Serial.println();
}

//3. Injection flow at 0.293ml/s===================================================================
void StepForwardHalf()
{
  Serial.println("Injection flow at 0.293ml/s");

for(x= 1; x<3; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    //Read direction pin state and change it
    state=digitalRead(dir);
    if(state == HIGH)
    {
      digitalWrite(dir, LOW);
    }
    
      digitalWrite(MS1, HIGH); //Pull MS1, and MS2 high to set logic to 1/8th microstep resolution
      digitalWrite(MS2, LOW);
      
    for(y=1; y<17100; y++)
    {
      digitalWrite(stp,HIGH); //Trigger one step
      delay(1);
      digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
      delay(1);
    }
  }
    
  Serial.println("Enter new option");
  Serial.println();
}

//4. Refill flow at 0.293ml/s===================================================================
void ReverseStepHalf()
{
  Serial.println("Refill flow at 0.293ml/s");

for(x= 1; x<3; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    //Read direction pin state and change it
    state=digitalRead(dir);
    digitalWrite(dir, HIGH);
    
    if(state == HIGH)
    {
      digitalWrite(dir, HIGH);
    }
    
      digitalWrite(MS1, HIGH); //Pull MS1, and MS2 high to set logic to 1/8th microstep resolution
      digitalWrite(MS2, LOW);
      
    for(y=1; y<17100; y++)
    {
      digitalWrite(stp,HIGH); //Trigger one step
      delay(1);
      digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
      delay(1);
    }
  }
    
  Serial.println("Enter new option");
  Serial.println();
}

//5. Injection flow at 0.146ml/s===================================================================
void StepForwardQuarter()
{
  Serial.println("Injection flow at 0.146ml/s");

for(x= 1; x<5; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    //Read direction pin state and change it
    state=digitalRead(dir);
    if(state == HIGH)
    {
      digitalWrite(dir, LOW);
    }
    
      digitalWrite(MS1, LOW); //Pull MS1, and MS2 high to set logic to 1/8th microstep resolution
      digitalWrite(MS2, HIGH);
      
    for(y=1; y<17100; y++)
    {
      digitalWrite(stp,HIGH); //Trigger one step
      delay(1);
      digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
      delay(1);
    }
  }
    
  Serial.println("Enter new option");
  Serial.println();
}

//6. Refill flow at 0.146ml/s===================================================================
void ReverseStepQuarter()
{
  Serial.println("Refill flow at 0.146ml/s");

for(x= 1; x<5; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    //Read direction pin state and change it
    state=digitalRead(dir);
    digitalWrite(dir, HIGH);
    
    if(state == HIGH)
    {
      digitalWrite(dir, HIGH);
    }
    
      digitalWrite(MS1, LOW); //Pull MS1, and MS2 high to set logic to 1/8th microstep resolution
      digitalWrite(MS2, HIGH);
      
    for(y=1; y<17100; y++)
    {
      digitalWrite(stp,HIGH); //Trigger one step
      delay(1);
      digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
      delay(1);
    }
  }
    
  Serial.println("Enter new option");
  Serial.println();
}

//7. Injection flow at 0.074ml/s===================================================================
void StepForwardSmallest()
{
  Serial.println("Injection flow at 0.074ml/s");

for(x= 1; x<9; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    //Read direction pin state and change it
    state=digitalRead(dir);
    if(state == HIGH)
    {
      digitalWrite(dir, LOW);
    }
    
      digitalWrite(MS1, HIGH); //Pull MS1, and MS2 high to set logic to 1/8th microstep resolution
      digitalWrite(MS2, HIGH);
      
    for(y=1; y<17100; y++)
    {
      digitalWrite(stp,HIGH); //Trigger one step
      delay(1);
      digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
      delay(1);
    }
  }
    
  Serial.println("Enter new option");
  Serial.println();
}

//8. Refill flow at 0.074ml/s===================================================================
void ReverseStepSmallest()
{
  Serial.println("Refill flow at 0.074ml/s");

for(x= 1; x<9; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    //Read direction pin state and change it
    state=digitalRead(dir);
    digitalWrite(dir, HIGH);
    
    if(state == HIGH)
    {
      digitalWrite(dir, HIGH);
    }
    
      digitalWrite(MS1, HIGH); //Pull MS1, and MS2 high to set logic to 1/8th microstep resolution
      digitalWrite(MS2, HIGH);
      
    for(y=1; y<17100; y++)
    {
      digitalWrite(stp,HIGH); //Trigger one step
      delay(1);
      digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
      delay(1);
    }
  }
    
  Serial.println("Enter new option");
  Serial.println();
}

//9. Forward/reverse stepping function
void CycledInjectRefill()
{
  Serial.println("Alternate between stepping Injecting and Refilling.");
  
  for(x= 1; x<5; x++)  //Loop the forward stepping enough times for motion to be visible
  {
    //Read direction pin state and change it
    state=digitalRead(dir);
    if(state == HIGH)
    {
      digitalWrite(dir, LOW);
    }
    
    else if(state ==LOW)
    {
      digitalWrite(dir,HIGH);
    }

    for(y=1; y<17000; y++)
    {
      digitalWrite(stp,HIGH); //Trigger one step
      delay(1);
      digitalWrite(stp,LOW); //Pull step pin low so it can be triggered again
      delay(1);
    }
  }
  Serial.println("Enter new option:");
  Serial.println();
}

//Reset Easy Driver pins to default states
void resetEDPins()
{
  digitalWrite(stp, LOW);
  digitalWrite(dir, LOW);
  digitalWrite(MS1, LOW);
  digitalWrite(MS2, LOW);
  digitalWrite(EN, HIGH);
}



